@@ -4,14 +4,13 @@ from math import ceil
 from sqlalchemy import orm, and_, or_, inspect
 from sqlalchemy.orm.strategy_options import Load
 
-DEFAULT_PER_PAGE = 50
-DEFAULT_SIMPLE_SEARCH_KEY = 'q'
-
 class Query(orm.Query):
 
     # when used as a query property (e.g. MyModel.query), then this is populated with the originating model
     __model__ = None
 
+    DEFAULT_PER_PAGE = 50
+
     @property
     def Model(self):
         '''Return originating model class when Query used a query property'''
@@ -122,14 +121,20 @@ class Query(orm.Query):
         '''Pluck `column` attribute values from `self.all()` results and return as list'''
         return [getattr(r, column, None) for r in self.all()]
 
-    def page(self, page=1, per_page=DEFAULT_PER_PAGE):
+    def page(self, page=1, per_page=None):
         '''Return query with limit and offset applied for page'''
+        if per_page is None:
+            per_page = self.DEFAULT_PER_PAGE
+
         return self.limit(per_page).offset((page - 1) * per_page)
 
-    def paginate(self, page=1, per_page=DEFAULT_PER_PAGE, error_out=True):
+    def paginate(self, page=1, per_page=None, error_out=True):
         if error_out and page < 1:
             raise IndexError
 
+        if per_page is None:
+            per_page = self.DEFAULT_PER_PAGE
+
         items = self.page(page, per_page).all()
 
         if not items and page != 1 and error_out:
@@ -151,8 +156,22 @@ class Query(orm.Query):
         filters = [m for m in [model.simple_search(search_string) for model in self.all_entities] if m is not None]
         return self.filter(or_(*filters)) if filters else self
 
-    def search(self, search_dict, simple_search_key=DEFAULT_SIMPLE_SEARCH_KEY):
-        return self.advanced_search(search_dict).simple_search(search_dict.get(simple_search_key, ''))
+    def search(self, search_string=None, search_dict=None, limit=None, offset=None):
+        query = self
+
+        if search_string is not None:
+            query = query.simple_search(search_string)
+
+        if search_dict is not None:
+            query = query.advanced_search(search_dict)
+
+        if limit is not None:
+            query = query.limit(limit)
+
+        if offset is not None:
+            query = query.offset(offset)
+
+        return query
 
 class Pagination(object):
     '''
